"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Platform = void 0;
require("reflect-metadata");
const accessories_1 = require("./accessories");
const sequence_1 = require("./accessories/sequence");
const settings_1 = require("./settings");
const tsyringe_1 = require("tsyringe");
const harmonyDataProvider_1 = require("./dataProviders/harmonyDataProvider");
const stateDataProvider_1 = require("./dataProviders/stateDataProvider");
const commandService_1 = require("./services/commandService");
const activityService_1 = require("./services/activityService");
const volumeService_1 = require("./services/volumeService");
class Platform {
    constructor(log, config, api) {
        this.log = log;
        this.api = api;
        this.Service = this.api.hap.Service;
        this.Characteristic = this.api.hap.Characteristic;
        // this is used to track restored cached accessories
        this.accessories = [];
        this.log.debug("Finished initializing platform:", config.name);
        this.config = config;
        this.register();
        let didFinishLaunching = false;
        this.api.on("didFinishLaunching", () => __awaiter(this, void 0, void 0, function* () {
            log.debug("Executed didFinishLaunching callback");
            this.discoverControlUnitAccessories();
            this.discoverDeviceButtonAccessories();
            this.discoverSequenceAccessories();
            this.pruneAccessories();
            didFinishLaunching = true;
        }));
    }
    /**
     * Discover new control unit accessories
     * @param dataProvider
     */
    discoverControlUnitAccessories() {
        this.config.ControlUnits.forEach((unit) => {
            const uuid = this.api.hap.uuid.generate(unit.DisplayName);
            const existingAccessory = this.accessories.find((e) => e.UUID === uuid);
            if (existingAccessory) {
                this.log.info("Restoring existing accessory from cache: " +
                    existingAccessory.displayName);
                new accessories_1.ControlUnit(this, existingAccessory, tsyringe_1.container.resolve(activityService_1.ActivityService), tsyringe_1.container.resolve(commandService_1.CommandService), tsyringe_1.container.resolve(volumeService_1.VolumeService), unit.Activities);
                this.api.publishExternalAccessories(settings_1.PLUGIN_NAME, [existingAccessory]);
                console.log("Publishing external accessory: " + uuid);
            }
            else {
                this.log.info("Adding new accessory: " + unit.DisplayName);
                const accessory = new this.api.platformAccessory(unit.DisplayName, uuid);
                accessory.context["DeviceName"] = unit.DisplayName;
                accessory.context["Type"] = typeof accessories_1.ControlUnit.name;
                new accessories_1.ControlUnit(this, accessory, tsyringe_1.container.resolve(activityService_1.ActivityService), tsyringe_1.container.resolve(commandService_1.CommandService), tsyringe_1.container.resolve(volumeService_1.VolumeService), unit.Activities);
                this.api.publishExternalAccessories(settings_1.PLUGIN_NAME, [accessory]);
                console.log("Publishing external accessory: " + uuid);
            }
        });
    }
    /**
     * Discover new device button accessories
     * @param dataProvider
     */
    discoverDeviceButtonAccessories() {
        this.config.DeviceButtons.forEach((button) => {
            const uuid = this.api.hap.uuid.generate(button.DisplayName);
            const existingAccessory = this.accessories.find((e) => e.UUID === uuid);
            if (existingAccessory) {
                this.log.info("Restoring existing accessory from cache: " +
                    existingAccessory.displayName);
                new accessories_1.DeviceButton(this, existingAccessory, tsyringe_1.container.resolve(harmonyDataProvider_1.HarmonyDataProvider), button);
                this.api.updatePlatformAccessories([existingAccessory]);
            }
            else {
                this.log.info("Adding new accessory: " + button.DisplayName);
                const accessory = new this.api.platformAccessory(button.DisplayName, uuid);
                accessory.context["DeviceName"] = button.DisplayName;
                accessory.context["Type"] = typeof accessories_1.DeviceButton;
                new accessories_1.DeviceButton(this, accessory, tsyringe_1.container.resolve(harmonyDataProvider_1.HarmonyDataProvider), button);
                this.api.registerPlatformAccessories(settings_1.PLUGIN_NAME, settings_1.PLATFORM_NAME, [
                    accessory,
                ]);
                this.accessories.push(accessory);
            }
        });
    }
    /**
     * Discover new sequence accessories
     * @param dataProvider
     */
    discoverSequenceAccessories() {
        this.config.Sequences.forEach((sequence) => {
            const uuid = this.api.hap.uuid.generate(sequence.DisplayName);
            const existingAccessory = this.accessories.find((e) => e.UUID === uuid);
            if (existingAccessory) {
                this.log.info("Restoring existing accessory from cache: " +
                    existingAccessory.displayName);
                new sequence_1.Sequence(this, existingAccessory, tsyringe_1.container.resolve(harmonyDataProvider_1.HarmonyDataProvider), sequence);
                this.api.updatePlatformAccessories([existingAccessory]);
            }
            else {
                this.log.info("Adding new accessory: " + sequence.DisplayName);
                const accessory = new this.api.platformAccessory(sequence.DisplayName, uuid);
                accessory.context["DeviceName"] = sequence.DisplayName;
                accessory.context["Type"] = typeof "Sequence";
                new sequence_1.Sequence(this, accessory, tsyringe_1.container.resolve(harmonyDataProvider_1.HarmonyDataProvider), sequence);
                this.api.registerPlatformAccessories(settings_1.PLUGIN_NAME, settings_1.PLATFORM_NAME, [
                    accessory,
                ]);
                this.accessories.push(accessory);
            }
        });
    }
    pruneAccessories() {
        // Remove devices not discovered
        const listOfConfiguredAccessories = [
            ...this.config.DeviceButtons.map((e) => e.DisplayName),
            ...this.config.Sequences.map((e) => e.DisplayName),
            ...this.config.ControlUnits.map((e) => e.DisplayName),
        ];
        for (const accessory of this.accessories) {
            if (listOfConfiguredAccessories.filter((displayName) => displayName === accessory.displayName).length === 0) {
                this.api.unregisterPlatformAccessories(settings_1.PLUGIN_NAME, settings_1.PLATFORM_NAME, [
                    accessory,
                ]);
            }
        }
    }
    configureAccessory(accessory) {
        this.log.info("Loading accessory from cache:", accessory.displayName);
        // add the restored accessory to the accessories cache so we can track if it has already been registered
        this.accessories.push(accessory);
    }
    register() {
        tsyringe_1.container.register(Platform, { useValue: this });
        tsyringe_1.container.register("IConfig", { useValue: this.config });
        tsyringe_1.container.register("log", { useValue: this.log });
        tsyringe_1.container.registerSingleton(harmonyDataProvider_1.HarmonyDataProvider);
        tsyringe_1.container.registerSingleton(activityService_1.ActivityService);
        tsyringe_1.container.registerSingleton(stateDataProvider_1.StateDataProvider);
        tsyringe_1.container.registerSingleton(commandService_1.CommandService);
    }
}
exports.Platform = Platform;
//# sourceMappingURL=platform.js.map